<?php

namespace App\Http\Controllers\API\v1;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\File;
use App\Models\ProductCategory;
use App\Models\ProductAttribute;
use App\Models\ProductImage;
use App\Models\ProductVariation;
use App\Models\VariationAttributeValue;
use App\Models\VariationImage;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use Throwable;

class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        DB::enableQueryLog();
        $store_id = $request->input('store_id', '');
        $keyword = $request->input('keyword', '');
        $perPage = $request->input('per_page');
        $sortBy = $request->input('sortBy', '');
        $sortType = $request->input('sortType', '');
        $keyword = str_replace(" ","%", $keyword);
        
        $data = Product::with(['images','categories','brand','variations'=> function($wv) {
            $wv->with(['images','attributes', 'stores']);
        }])
        ->where(function ($query) use ($keyword ) {
            $query->where('title', 'like', '%'.$keyword.'%')
                ->orWhereHas('variations', function($q) use ($keyword ) {
                    $q->where('product_no', 'like', '%'.$keyword.'%')
                    ->orWhere('sku', 'like', '%'.$keyword.'%')
                    ;
                })
                ;
        });
        
        if (!empty($sortBy) && !empty($sortType)) {
            $data = $data->orderBy($sortBy, $sortType);
        }
        // $data = $data->get();
        // dd(DB::getQueryLog());
        $data = $data->paginate($perPage);
        return response()->json($data);
    }

    /**
     * Product a newly created resource in storage.
     */
    public function store(Request $request)
    {
        // Validate the incoming request data
        $validator = Validator::make($request->all(), [
            'title' => 'required|min:2|unique:products,title',
        ]);
        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], Response::HTTP_UNPROCESSABLE_ENTITY);
        }
        $data = $request->validate([
            'store_id' => 'required|string',
            'title' => 'required|string',
            'description' => 'nullable',
            'published' => 'string',
            'published_at' => 'nullable',
            'categories' => 'array',
            'brand_id' => 'string',
            'image' => 'nullable',
            'images' => 'nullable',
            'attributes' => 'array',
            'variations' => 'required|array',
        ]);
        if(!$request->variations || count($request->variations)==0) {
            return response()->json(['errors' => ['Please input details on variation section']], Response::HTTP_UNPROCESSABLE_ENTITY);
        }

        DB::connection()->beginTransaction();
        
        try {
            /**Create product record */
            $product = Product::firstOrCreate(
                [
                    'title' => $data['title']
                ],
                array_merge(
                    $request->except(['variations','categories', 'attributes'],
                    [
                        'created_at' => $this->current_datetime,
                        'updated_at' => $this->current_datetime,
                    ]
                ))
            );
            /**if categories is provided */
            // dd($request);
            if($request->categories && !empty($request->categories)) {
                foreach($request->categories as $category_id) {
                    ProductCategory::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'category_id' => $category_id
                        ],
                        [
                        'product_id' => $product->id,
                        'category_id' => $category_id,
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
                
            /**if categories is provided */
            if($request->attributes && !empty($request->attributes)) {
                foreach($request->attributes as $attribute_id) {
                    ProductAttribute::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'attribute_id' => $attribute_id,
                        ],
                        [
                        'product_id' => $product->id,
                        'attribute_id' => $attribute_id,
                        'product_id' => $product->id,
                        'attribute_id' => $attribute_id,
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
            
            /**set image path */
            $image_path = 'products';
            /**if main image is uploaded */
            if ($request->hasFile('image')) {
                $image = $request->file('image');
                $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                $filePath = Storage::putFileAs($image_path, $image, $imageName);
                $file = File::create([
                    'filename' => $imageName,
                    'disk' => 'storage',
                    'path' => $filePath,
                    'extension' => $image->getClientOriginalExtension(),
                    'size' => $image->getSize(),
                    'mime' => $image->getMimeType(),
                ]);
                ProductImage::create([
                    'product_id' => $product->id, 
                    'file_id' => $file->id, 
                    'default' => true,
                    'created_at' => $product->created_at,
                    'updated_at' => $product->updated_at,
                ]);
            }
            /**if has uploaded image */
            if ($request->hasFile('images')) {
                foreach($request->file('images') as $image) {
                    $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                    $filePath = Storage::putFileAs($image_path, $image, $imageName);
                    $file = File::create([
                        'filename' => $imageName,
                        'disk' => 'storage',
                        'path' => $filePath,
                        'extension' => $image->getClientOriginalExtension(),
                        'size' => $image->getSize(),
                        'mime' => $image->getMimeType(),
                    ]);
                    ProductImage::create([
                        'product_id' => $product->id, 
                        'file_id' => $file->id, 
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
            /**variations */
            if($request->variations) {
                /**loop through each variation */
                // dd($request->variations);
                foreach($request->variations as $var) {
                    /**create variation */
                    $variation = ProductVariation::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'product_no' => $var['product_no'] ?? "",
                        ],
                        [
                            'product_id' => $product->id,
                            'product_no' => $var['product_no'] ?? "",
                            'sku' => $var['sku'] ?? "",
                            'price' => $var['price'] ?? 0,
                            'length' => $var['length'],
                            'width' => $var['width'],
                            'height' => $var['height'],
                            'weight' => $var['weight'],
                            'published' => $product->published,
                            'published_at' => $product->published_at,
                            'created_at' => $product->created_at,
                            'upated_at' => $product->updated_at,
                        ]
                    );
                    /**Add store product  */
                    $storeProduct = \App\Models\StoreProductVariation::firstOrCreate(
                        [
                            'store_id' => $data['store_id'],
                            'product_variation_id' => $variation->id
                        ],
                        [
                            'store_id' => $data['store_id'],
                            'product_variation_id' => $variation->id
                        ]);

                    /**if has variation attributes */
                    if($var['attributes'] && !empty($var['attributes'])) {
                        foreach($var['attributes'] as $attr) {
                            /**create variation attribute record */
                            $attribute = VariationAttributeValue::create([
                                'variation_id' => $variation->id,
                                'attribute_id' => $attr['id'],
                                'value' => $attr['value'],
                                'created_at' => $product->created_at,
                                'updated_at' => $product->updated_at,
                            ]);
                        }
                    }
                    /** if has variation image */
                    // dd($var['image']->getClientOriginalExtension());
                    if($var['image']) {
                        // Process the file since it exists and is valid
                        $image = $var['image'];
                        $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                        $filePath = Storage::putFileAs($image_path, $image, $imageName);
                        $file = File::create([
                            'filename' => $imageName,
                            'disk' => 'storage',
                            'path' => $filePath,
                            'extension' => $image->getClientOriginalExtension(),
                            'size' => $image->getSize(),
                            'mime' => $image->getMimeType(),
                        ]);
                        VariationImage::create([
                            'variation_id' => $variation->id, 
                            'file_id' => $file->id, 
                            'created_at' => $product->created_at,
                            'updated_at' => $product->updated_at,
                        ]);

                    }
                    if($var['inventories'] && count($var['inventories'])>0) {
                        foreach($var['inventories'] as $inventory) {
                            $spbl = \App\Models\StoreProductCountByLocation::where([
                                'store_id' => $data['store_id'],
                                'location_id' => $inventory['location_id'],
                                'variation_id' => $variation->id,
                            ])->first();
                            $inv = \App\Models\Inventory::where(['store_id' => $data['store_id'],'variation_id'=>$variation->id])->first();
                            $old_qty = 0;
                            $new_qty = 0;
                            if($inv) {
                                $old_qty = $inv->quantity;
                            }
                            if($spbl) {
                                $spbl->increment('quantity', $inventory['quantity'], ['updated_at' => $variation->updated_at]);
                            } else {
                                $spbl = \App\Models\StoreProductCountByLocation::create([
                                    'store_id' => $data['store_id'],
                                    'location_id' => $inventory['location_id'],
                                    'variation_id' => $variation->id,
                                    'quantity' => $inventory['quantity'],
                                    'created_at' => $variation->created_at,
                                    'updated_at' => $variation->updated_at,
                                ]);
                            }
                            $new_qty = $old_qty + $inventory['quantity'];

                            \App\Models\StockCard::create([
                                'store_id' => $data['store_id'],
                                'variation_id' => $variation->id,
                                'progress' => 'completed',
                                'quantity' => $inventory['quantity'],
                                'quantity_new' => $new_qty,
                                'quantity_old' => $old_qty,
                                'type' => 'in',
                                'remarks' => 'default initial setup',
                                'created_at' => $variation->created_at,
                                'updated_at' => $variation->updated_at,
                            ]);
                        }
                    }
                    // if($var->hasFile('image_src')) {
                    //     $image = $var->file('image_src');
                        
                    // }
                }
            }
            DB::connection()->commit();

            return response()->json($product, 200);
            
        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ], Response::HTTP_UNPROCESSABLE_ENTITY);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(int $id)
    {
        $data = Product::find($id);
        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }
        return response()->json($data);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, int $id)
    {
        $data = Product::find($id);

        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }

        DB::connection()->beginTransaction();

        try {
        
            $data->fill($request->all());
            $data->save();

            $data->touch();

            DB::connection()->commit();

            return response()->json($data);

        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $data = Product::find($id);

        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }

        DB::connection()->beginTransaction();

        try {
            $data->delete();

            DB::connection()->commit();

            return response()->json([
                'status'  => true,
                'message' => 'Record successfully deleted.',
                'data'    => $data,
            ]);

        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }
}
