<?php

namespace App\Http\Controllers\API\v1;

use App\Http\Controllers\Controller;
use App\Models\Module;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Throwable;

class ModuleController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        $perPage = $request->input('per_page');

        $keyword = $request->input('keyword', '');

        $data = Module::with(['menu'])->where(function ($query) use ($keyword) {
            $query->where('name', 'like', '%'.$keyword.'%');
        })->orderBy('name')->paginate($perPage);

        return response()->json($data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {

        $validator = Validator::make($request->only(['name']), [
            'name' => "required|unique:modules,name"
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            $request_data = array_merge($request->all(), ['created_at'    => $this->current_datetime,
            'updated_at'    => $this->current_datetime]);

            $module = Module::create($request_data);

            DB::connection()->commit();

            return response()->json([
                'status'  => true,
                'message' => 'Saved successfully!',
                'data'    => $module
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    public function show($id)
    {
        return response()->json([
            'status'  => true,
            'message' => 'Module info',
            'data'    => Module::find($id)
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validator = Validator::make(array_merge($request->only(['name']), ['id' => $id]), [
            'id'   => 'required|exists:modules,id',
            'name' => 'required|unique:modules,name,'.$id
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            // update
            $module = Module::find($id);

            $request_data = array_merge($request->all(), ['updated_at'    => $this->current_datetime]);

            $module->fill($request_data);
            $module->save();

            $module->touch();

            DB::connection()->commit();

            return response()->json([
                'status'  => true,
                'message' => 'Updated successfully!',
                'data'    => $module
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }
}
