<?php

namespace App\Http\Controllers\API\v1;

use App\Http\Controllers\Controller;
use App\Models\Permission;
use App\Models\RolePermission;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Throwable;

class PermissionController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     */
    public function index(Request $request)
    {

        $perPage = $request->input('per_page');

        $keyword = $request->input('keyword', '');

        $data = Permission::where(function ($query) use ($keyword) {
            $query->where('code', 'like', '%'.$keyword.'%')
            ->orWhere('name', 'like', '%'.$keyword.'%')
            ->orWhere('description', 'like', '%'.$keyword.'%');
        })->orderBy('name')->paginate($perPage);

        return response()->json($data);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->only(['code', 'name']), [
            'code' => "required|max:20|unique:permissions,code",
            'name' => "required|unique:permissions,name"
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {
            $permission = Permission::create([
                'code'          => $request->code,
                'name'          => strtoupper($request->name),
                'description'          => $request->description,
                'active'        => $request->active,
                'created_by_id' => $request->created_by_id,
                'created_at'    => $this->current_datetime,
                'updated_at'    => $this->current_datetime
            ]);

            DB::connection()->commit();

            return response()->json([
                'status'  => true,
                'message' => 'Saved successfully!',
                'data'    => $permission
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        return response()->json([
            'status'  => true,
            'message' => 'Permission info',
            'data'    => Permission::find($id)
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $validator = Validator::make(array_merge($request->only(['code', 'name']), ['id' => $id]), [
            'id'   => 'required|exists:permissions,id',
            'code' => 'required|max:20|unique:permissions,code,'.$id,
            'name' => 'required|unique:permissions,name,'.$id
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            // update
            $permission = Permission::find($id);

            $permission->code       = $request->code;
            $permission->name       = strtoupper($request->name);

            if($request->description)
                $permission->description       = $request->description;

            if($request->is_active)
                $permission->is_active  = $request->is_active;


            $permission->updated_at = $this->current_datetime;
            $permission->save();

            DB::connection()->commit();

            return response()->json([
                'status'  => true,
                'message' => 'Updated successfully!',
                'data'    => $permission
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    public function SaveRolePermissions(Request $request) {

        $validator = Validator::make($request->only(['id']), [
            'id' => "required|exists:roles,id"
        ]);

        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            //check if has role permissions
            $RolePermission = RolePermission::where('role_id', $request->id);
            $RolePermission->delete();

            $role_permission = [];
            foreach($request->stores as $store) {
                foreach($store['modules'] as $module) {
                    foreach($module['permissions'] as $permission) {
                        $role_permission[] = array(
                            'role_id'=>$request->id,
                            'store_id'=>$store['id'],
                            'module_id'=>$module['id'],
                            'permission_id'=>$permission['id'],
                            'created_at'=>$this->current_datetime,
                            'updated_at'=>$this->current_datetime,
                        );
                    }
                }
            }

            RolePermission::insert($role_permission);

            DB::connection()->commit();

            return response()->json([
                'message' => 'Saved successfully!',
                'data'    => $request->id
            ]);


        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ], Response::HTTP_UNPROCESSABLE_ENTITY);
        }

    }

    function getAllRolePermissions(Request $request) {

        try {
            $rolePermission = RolePermission::all();
            return response()->json([
                'status'  => true,
                'data'    => $rolePermission
            ]);
        } catch (Throwable $e) {
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    function getRolePermissions(string $role_id) {

        try {
            $rolePermission = RolePermission::where('role_id', $role_id)->get();
            return response()->json([
                'data'    => $rolePermission
            ],200);

        } catch (Throwable $e) {
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    function StorePermissions(Request $request) {
        
        try {
            DB::enableQueryLog();
            $rolePermission = RolePermission::whereIn('role_id', $request->roles)
                ->where(function($query) use ($request) {
                    $query->where('store_id', $request->store_id)->orWhere('store_id','0');
                })
                ->with(['module','permission'])
                ->get();
            return response()->json([
                'data'    => $rolePermission
            ]);
          
        } catch (Throwable $e) {
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

}
