<?php

namespace App\Http\Controllers\API\v1;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\File;
use App\Models\ProductCategory;
use App\Models\ProductAttribute;
use App\Models\ProductImage;
use App\Models\Variation;
use App\Models\VariationAttributeValue;
use App\Models\VariationImage;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
use Throwable;

class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        DB::enableQueryLog();
        $store_id = $request->input('store_id', '');
        $keyword = $request->input('keyword', '');
        $perPage = $request->input('per_page');
        $sortBy = $request->input('sortBy', '');
        $sortType = $request->input('sortType', '');
        $keyword = str_replace(" ","%", $keyword);
        $data = Product::with(['images','categories','brand','variations'=> function($wv) {
            $wv->with(['images','attributes', 'store_details']);
        }])
        ->where(function ($query) use ($keyword) {
            $query->where('title', 'like', '%'.$keyword.'%')
                ->orWhereHas('variations', function($q) use ($keyword) {
                    $q->where('product_no', 'like', '%'.$keyword.'%')
                    ->orWhere('sku', 'like', '%'.$keyword.'%')
                    ;
                })
                ;
        });
        
        if (!empty($sortBy) && !empty($sortType)) {
            $data = $data->orderBy($sortBy, $sortType);
        }
        // $data = $data->get();
        // dd(DB::getQueryLog());
        $data = $data->paginate($perPage);
        return response()->json($data);
    }

     /**
     * Display a listing of the searched resource.
     */
    public function all(Request $request)
    {
        DB::enableQueryLog();
        $store_id = $request->input('store_id', null);
        $keyword = $request->input('keyword', '');
        $perPage = $request->input('per_page');
        $sortBy = $request->input('sortBy', '');
        $sortType = $request->input('sortType', '');
        $keyword = str_replace(" ","%", $keyword);
        $data = Product::with(['images','variations'=> function($wv) use ($store_id) {
            $wv->with(['attributes', 'inventories' => function($queryi) use ($store_id) {
                $queryi->where('store_id', '=', $store_id);
            }]);
        }])
        ;
        $data = $data->orderBy('title');
        // if (!empty($sortBy) && !empty($sortType)) {
            // }
            // $data = $data->get();
            $data = $data->get();
            // dd(DB::getQueryLog());
            // dd($data);
        return response()->json($data);
    }

    /**
     * Display the specified resource.
     */
    public function show(Request $request, int $id)
    {
        $store_id = $request->input('store_id', null);
        $data = Product::with(['images', 'brand', 'categories', 'attributes','variations' => function($query) use ($store_id) {
            $query->with([
                'attributes', 
                'store_details' => function($querys) use ($store_id) {
                    if($store_id !== null) {
                        $querys->where('store_id', '=', $store_id)->first();
                    }
                },
                'inventories' => function($queryi) use ($store_id) {
                    if($store_id !== null) {
                        $queryi->where('store_id', '=', $store_id);
                    }
                },
                'stock_cards' => function($queryc) use ($store_id) {
                    if($store_id !== null) {
                        $queryc->where('store_id', '=', $store_id);
                    }
                }
            ]);
        }])->find($id);
        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }
        return response()->json($data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $data = $request->validate([
            'store_id' => 'required|string',
            'title' => 'required|string',
            'description' => 'nullable',
            'published' => 'string',
            'published_at' => 'nullable',
            'categories' => 'array',
            'brand_id' => 'string',
            'image' => 'nullable',
            'images' => 'nullable',
            'attributes' => 'array',
            'variations' => 'required|array',
        ]);
        if(!$request->variations || count($request->variations)==0) {
            return response()->json(['errors' => ['Please input details on variation section']], Response::HTTP_UNPROCESSABLE_ENTITY);
        }

        DB::connection()->beginTransaction();
        
        try {
            /**Create product record */
            $product = Product::firstOrCreate(
                [
                    'title' => $data['title']
                ],
                array_merge(
                    $request->except(['variations','categories', 'attributes'],
                    [
                        'created_at' => $this->current_datetime,
                        'updated_at' => $this->current_datetime,
                    ]
                ))
            );
            /**if categories is provided */
            // dd($request);
            if($request->categories && !empty($request->categories)) {
                foreach($request->categories as $category_id) {
                    ProductCategory::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'category_id' => $category_id
                        ],
                        [
                        'product_id' => $product->id,
                        'category_id' => $category_id,
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
                
            /**if categories is provided */
            if($request->attributes && !empty($request->attributes)) {
                foreach($request->attributes as $attribute_id) {
                    ProductAttribute::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'attribute_id' => $attribute_id,
                        ],
                        [
                        'product_id' => $product->id,
                        'attribute_id' => $attribute_id,
                        'product_id' => $product->id,
                        'attribute_id' => $attribute_id,
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
            
            /**set image path */
            $image_path = 'products';
            /**if main image is uploaded */
            if ($request->hasFile('image')) {
                $image = $request->file('image');
                $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                $filePath = Storage::putFileAs($image_path, $image, $imageName);
                $file = File::create([
                    'filename' => $imageName,
                    'disk' => 'storage',
                    'path' => $filePath,
                    'extension' => $image->getClientOriginalExtension(),
                    'size' => $image->getSize(),
                    'mime' => $image->getMimeType(),
                ]);
                ProductImage::create([
                    'product_id' => $product->id, 
                    'file_id' => $file->id, 
                    'default' => true,
                    'created_at' => $product->created_at,
                    'updated_at' => $product->updated_at,
                ]);
            }
            /**if has uploaded image */
            if ($request->hasFile('images')) {
                foreach($request->file('images') as $image) {
                    $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                    $filePath = Storage::putFileAs($image_path, $image, $imageName);
                    $file = File::create([
                        'filename' => $imageName,
                        'disk' => 'storage',
                        'path' => $filePath,
                        'extension' => $image->getClientOriginalExtension(),
                        'size' => $image->getSize(),
                        'mime' => $image->getMimeType(),
                    ]);
                    ProductImage::create([
                        'product_id' => $product->id, 
                        'file_id' => $file->id, 
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
            /**variations */
            if($request->variations) {
                /**loop through each variation */
                // dd($request->variations);
                foreach($request->variations as $var) {
                    /**create variation */
                    $variation = Variation::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'product_no' => $var['product_no'] ?? "",
                        ],
                        [
                            'product_id' => $product->id,
                            'product_no' => $var['product_no'] ?? "",
                            'sku' => $var['sku'] ?? "",
                            'default_price' => $var['price'] ?? 0,
                            'length' => $var['length'],
                            'width' => $var['width'],
                            'height' => $var['height'],
                            'weight' => $var['weight'],
                            'max_qty' => $var['max_qty'],
                            'min_qty' => $var['min_qty'],
                            'reorder_qty' => $var['reorder_qty'],
                            'default' => $var['default'],
                            'published' => $product->published,
                            'published_at' => $product->published_at,
                            'created_at' => $product->created_at,
                            'upated_at' => $product->updated_at,
                            ]
                        );
                        /**Add store product  */
                        $storeProduct = \App\Models\StoreProductDetails::firstOrCreate(
                            [
                                'store_id' => $data['store_id'],
                                'product_id' => $product->id,
                                'variation_id' => $variation->id,
                            ],
                            [
                                'store_id' => $data['store_id'],
                                'product_id' => $product->id,
                                'variation_id' => $variation->id,
                                'reorder_qty' => $var['reorder_qty'],
                                'enabled' => true,
                                'default' => $var['default'],
                        ]);
                        /**Add price */
                        \App\Models\ProductPrice::create([
                            'store_id' => $data['store_id'],
                            'product_id' => $variation->id,
                            'variation_id' => $variation->id,
                            'price' => $var['price'],
                            'effectivity_date' => $product->published_at,
                            'created_at' => $product->created_at,
                            'updated_at' => $product->updated_at,
                        ]);
                        /**Add cost if any */
                        if($var['cost']) {
                            \App\Models\ProductCost::create([
                                'store_id' => $data['store_id'],
                                'product_id' => $variation->id,
                                'variation_id' => $variation->id,
                                'cost' => $var['cost'],
                                'effectivity_date' => $product->published_at,
                                'created_at' => $product->created_at,
                                'updated_at' => $product->updated_at,
                            ]);
                        }

                    /**if has variation attributes */
                    if($var['attributes'] && !empty($var['attributes'])) {
                        foreach($var['attributes'] as $attr) {
                            /**create variation attribute record */
                            $attribute = VariationAttributeValue::create([
                                'variation_id' => $variation->id,
                                'attribute_id' => $attr['id'],
                                'value' => $attr['value'],
                                'created_at' => $product->created_at,
                                'updated_at' => $product->updated_at,
                            ]);
                        }
                    }
                    /** if has variation image */
                    // dd($var['image']->getClientOriginalExtension());
                    if($var['image']) {
                        // Process the file since it exists and is valid
                        $image = $var['image'];
                        $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                        $filePath = Storage::putFileAs($image_path, $image, $imageName);
                        $file = File::create([
                            'filename' => $imageName,
                            'disk' => 'storage',
                            'path' => $filePath,
                            'extension' => $image->getClientOriginalExtension(),
                            'size' => $image->getSize(),
                            'mime' => $image->getMimeType(),
                        ]);
                        VariationImage::create([
                            'variation_id' => $variation->id, 
                            'file_id' => $file->id, 
                            'created_at' => $product->created_at,
                            'updated_at' => $product->updated_at,
                        ]);

                    }
                    if($var['inventories'] && count($var['inventories'])>0) {
                        foreach($var['inventories'] as $inventory) {
                            $spbl = \App\Models\StoreProductCountByLocation::where([
                                'store_id' => $data['store_id'],
                                'location_id' => $inventory['location_id'],
                                'variation_id' => $variation->id,
                            ])->first();
                            $inv = \App\Models\Inventory::where(['store_id' => $data['store_id'],'variation_id'=>$variation->id])->first();
                            $old_qty = 0;
                            $new_qty = 0;
                            if($inv) {
                                $old_qty = $inv->quantity;
                            }
                            if($spbl) {
                                $spbl->increment('quantity', $inventory['quantity'], ['updated_at' => $variation->updated_at]);
                            } else {
                                $spbl = \App\Models\StoreProductCountByLocation::create([
                                    'store_id' => $data['store_id'],
                                    'location_id' => $inventory['location_id'],
                                    'variation_id' => $variation->id,
                                    'quantity' => $inventory['quantity'],
                                    'created_at' => $variation->created_at,
                                    'updated_at' => $variation->updated_at,
                                ]);
                            }
                            $new_qty = $old_qty + $inventory['quantity'];

                            \App\Models\StockCard::create([
                                'store_id' => $data['store_id'],
                                'variation_id' => $variation->id,
                                'progress' => 'completed',
                                'quantity' => $inventory['quantity'],
                                'quantity_new' => $new_qty,
                                'quantity_old' => $old_qty,
                                'type' => 'in',
                                'remarks' => 'default initial setup',
                                'created_at' => $variation->created_at,
                                'updated_at' => $variation->updated_at,
                            ]);
                        }
                    }
                    // if($var->hasFile('image_src')) {
                    //     $image = $var->file('image_src');
                        
                    // }
                }
            }
            DB::connection()->commit();

            return response()->json($product, 200);
            
        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ], Response::HTTP_UNPROCESSABLE_ENTITY);
        }
    }
    /**
     * Store a newly created resource in storage.
     */
    public function import(Request $request)
    {
        DB::connection()->beginTransaction();
        $imported_data = $request->all();
        $store_id = $imported_data['store_id'];
        // dd($imported_data['products']);
        try {
            foreach($imported_data['products'] as $data) {
                // dd($data);
                $variations = $data['variations'];
                $categories = $data['categories'];
                $brand_name = $data['brand'];
                $attributes = $data['attributes'];
                $images = $data['images'];
                unset($data['variations']);
                unset($data['categories']);
                unset($data['brand']);
                unset($data['attributes']);
                unset($data['images']);
                /**Create product record */
                $brand = \App\Models\Brand::firstOrCreate(
                    [
                        'name' => $brand_name,
                    ],
                    [
                        'name' => $brand_name,
                        'created_at' => $this->current_datetime,
                        'updated_at' => $this->current_datetime,
                    ],
                );
                $data['brand_id'] = $brand->id;
                $product = Product::firstOrCreate(
                    [
                        'title' => $data['title']
                    ],
                    array_merge(
                        $data,
                        [
                            'created_at' => $this->current_datetime,
                            'updated_at' => $this->current_datetime,
                        ]
                    )
                );
                /**if categories is provided */
                // dd($request);
                if(!empty($categories)) {
                    foreach($categories as $category_name) {
                        $category = \App\Models\Category::firstOrCreate(
                            [
                                'name' => $category_name
                            ],
                            [
                                'name' => $category_name,
                                'created_at' => $product->created_at,
                                'updated_at' => $product->updated_at,
                            ]
                        );
                        ProductCategory::firstOrCreate(
                            [
                                'product_id' => $product->id,
                                'category_id' => $category->id
                            ],
                            [
                            'product_id' => $product->id,
                            'category_id' => $category->id,
                            'created_at' => $product->created_at,
                            'updated_at' => $product->updated_at,
                        ]);
                    }
                }
                    
                /**if categories is provided */
                if(!empty($attributes)) {
                    foreach($attributes as $attribute_name) {
                        $attribute = \App\Models\Attribute::firstOrCreate(
                            [
                                'name' => $attribute_name
                            ],
                            [
                                'name' => $attribute_name,
                                'created_at' => $product->created_at,
                                'updated_at' => $product->updated_at,
                            ],
                        );
                        ProductAttribute::firstOrCreate(
                            [
                                'product_id' => $product->id,
                                'attribute_id' => $attribute->id,
                            ],
                            [
                            'product_id' => $product->id,
                            'attribute_id' => $attribute->id,
                            'product_id' => $product->id,
                            'attribute_id' => $attribute->id,
                            'created_at' => $product->created_at,
                            'updated_at' => $product->updated_at,
                        ]);
                    }
                }
                
                /**set image path */
                $image_path = 'products';
                /**if main image is uploaded */
                // if ($request->hasFile('image')) {
                //     $image = $request->file('image');
                //     $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                //     $filePath = Storage::putFileAs($image_path, $image, $imageName);
                //     $file = File::create([
                //         'filename' => $imageName,
                //         'disk' => 'storage',
                //         'path' => $filePath,
                //         'extension' => $image->getClientOriginalExtension(),
                //         'size' => $image->getSize(),
                //         'mime' => $image->getMimeType(),
                //     ]);
                //     ProductImage::create([
                //         'product_id' => $product->id, 
                //         'file_id' => $file->id, 
                //         'default' => true,
                //         'created_at' => $product->created_at,
                //         'updated_at' => $product->updated_at,
                //     ]);
                // }
                // /**if has uploaded image */
                // if ($request->hasFile('images')) {
                //     foreach($request->file('images') as $image) {
                //         $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                //         $filePath = Storage::putFileAs($image_path, $image, $imageName);
                //         $file = File::create([
                //             'filename' => $imageName,
                //             'disk' => 'storage',
                //             'path' => $filePath,
                //             'extension' => $image->getClientOriginalExtension(),
                //             'size' => $image->getSize(),
                //             'mime' => $image->getMimeType(),
                //         ]);
                //         ProductImage::create([
                //             'product_id' => $product->id, 
                //             'file_id' => $file->id, 
                //             'created_at' => $product->created_at,
                //             'updated_at' => $product->updated_at,
                //         ]);
                //     }
                // }
                /**variations */
                if(!empty($variations)) {
                    /**loop through each variation */
                    // dd($request->variations);
                    foreach($variations as $var) {
                        /**create variation */
                        $variation = Variation::firstOrCreate(
                            [
                                'product_id' => $product->id,
                                'product_no' => $var['product_no'] ?? "",
                            ],
                            [
                                'product_id' => $product->id,
                                'product_no' => $var['product_no'] ?? "",
                                'sku' => $var['sku'] ?? "",
                                'default_price' => $var['price'] ?? 0,
                                'length' => $var['length'] ?? 0,
                                'width' => $var['width'] ?? 0,
                                'height' => $var['height'] ?? 0,
                                'weight' => $var['weight'] ?? 0,
                                'max_qty' => $var['max_qty'] ?? 0,
                                'min_qty' => $var['min_qty'] ?? 0,
                                'reorder_qty' => $var['min_qty'] ?? 0,
                                'default' => $var['default'] ?? false,
                                'published' => $product->published,
                                'published_at' => $product->published_at,
                                'created_at' => $product->created_at,
                                'upated_at' => $product->updated_at,
                                ]
                            );
                            /**Add store product  */
                            $storeProduct = \App\Models\StoreProductDetails::firstOrCreate(
                                [
                                    'store_id' => $store_id,
                                    'product_id' => $product->id,
                                    'variation_id' => $variation->id,
                                ],
                                [
                                    'store_id' => $store_id,
                                    'product_id' => $product->id,
                                    'variation_id' => $variation->id,
                                    'reorder_qty' => $var['min_qty'] ?? 0,
                                    'enabled' => true,
                                    'default' => $var['default'],
                            ]);
                            /**Add price */
                            \App\Models\ProductPrice::create([
                                'store_id' => $store_id,
                                'product_id' => $product->id,
                                'variation_id' => $variation->id,
                                'price' => $var['price'],
                                'effectivity_date' => $product->published_at ?? $product->created_at,
                                'created_at' => $product->created_at,
                                'updated_at' => $product->updated_at,
                            ]);
                            /**Add cost if any */
                            if(isset($var['cost'])) {
                                \App\Models\ProductCost::create([
                                    'store_id' => $store_id,
                                    'product_id' => $product->id,
                                    'variation_id' => $variation->id,
                                    'cost' => $var['cost'] ?? 0,
                                    'effectivity_date' => $product->published_at ?? $product->created_at,
                                    'created_at' => $product->created_at,
                                    'updated_at' => $product->updated_at,
                                ]);
                            }
    
                        /**if has variation attributes */
                        if($var['attributes'] && !empty($var['attributes'])) {
                            foreach($var['attributes'] as $attr) {
                                /**create variation attribute record */
                                $attribute = \App\Models\Attribute::firstOrCreate(
                                    [
                                        'name' => $attr['attribute_name'],
                                    ],
                                    [
                                        'name' => $attr['attribute_name'],
                                        'created_at' => $product->created_at,
                                        'updated_at' => $product->updated_at,
                                    ],
                                );
                                VariationAttributeValue::create([
                                    'variation_id' => $variation->id,
                                    'attribute_id' => $attribute->id,
                                    'value' => $attr['value'],
                                    'created_at' => $product->created_at,
                                    'updated_at' => $product->updated_at,
                                ]);
                            }
                        }
                        /** if has variation image */
                        // dd($var['image']->getClientOriginalExtension());
                        // if($var['image']) {
                        //     // Process the file since it exists and is valid
                        //     $image = $var['image'];
                        //     $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                        //     $filePath = Storage::putFileAs($image_path, $image, $imageName);
                        //     $file = File::create([
                        //         'filename' => $imageName,
                        //         'disk' => 'storage',
                        //         'path' => $filePath,
                        //         'extension' => $image->getClientOriginalExtension(),
                        //         'size' => $image->getSize(),
                        //         'mime' => $image->getMimeType(),
                        //     ]);
                        //     VariationImage::create([
                        //         'variation_id' => $variation->id, 
                        //         'file_id' => $file->id, 
                        //         'created_at' => $product->created_at,
                        //         'updated_at' => $product->updated_at,
                        //     ]);
                            
                        // }
                        if(isset($var['inventories']) && count($var['inventories'])>0) {
                            foreach($var['inventories'] as $inventory) {
                                /**SETUP LOCATION */
                                $warehouse = \App\Models\Warehouse::firstOrCreate(
                                    [
                                        'name' => $inventory['location']['warehouse'],
                                    ],
                                    [
                                        'name' => $inventory['location']['warehouse'],
                                        'created_at' => $product->created_at,
                                        'updated_at' => $product->updated_at,
                                    ]
                                );
                                
                                $floor = isset($inventory['location']['floor']) && !empty($inventory['location']['floor']) 
                                    ? 
                                        \App\Models\Floor::firstOrCreate(
                                            [
                                                'name' => $inventory['location']['floor']
                                            ],
                                            [
                                                'name' => $inventory['location']['floor'],
                                                'created_at' => $product->created_at,
                                                'updated_at' => $product->updated_at,
                                            ]
                                        ) 
                                    : null ; 
                                $rack = isset($inventory['location']['rack']) && !empty($inventory['location']['rack'])
                                    ?
                                        \App\Models\Rack::firstOrCreate(
                                            [
                                                'name' => $inventory['location']['rack,']
                                            ],
                                            [
                                                'name' => $inventory['location']['rack'],
                                                'created_at' => $product->created_at,
                                                'updated_at' => $product->updated_at,
                                            ],
                                        )
                                    : null ;
                                $bin = isset($inventory['location']['bin']) && !empty($inventory['location']['bin'])
                                    ?
                                        \App\Models\Bin::firstOrCreate(
                                            [
                                                'name' => $inventory['location']['bin'],
                                            ],
                                            [
                                                'name' => $inventory['location']['bin'],
                                                'created_at' => $product->created_at,
                                                'updated_at' => $product->updated_at,
                                            ],
                                        )
                                    : null ;
                                $inventory_location = \App\Models\InventoryLocation::firstOrCreate(
                                    [
                                        'store_id' => $store_id,
                                        'warehouse_id' => $warehouse->id,
                                        'floor_id' => $floor!== null ? $floor->id : null,
                                        'rack_id' => $rack!==null ? $rack->id : null,
                                        'bin_id' => $bin!==null ? $bin->id : null,
                                    ],
                                    [
                                        'store_id' => $store_id,
                                        'warehouse_id' => $warehouse->id,
                                        'floor_id' => $floor!== null ? $floor->id : null,
                                        'rack_id' => $rack!==null ? $rack->id : null,
                                        'bin_id' => $bin!==null ? $bin->id : null,
                                        'created_at' => $product->created_at,
                                        'updated_at' => $product->updated_at,
                                    ],
                                );
                                $spbl = \App\Models\StoreProductCountByLocation::where([
                                    'store_id' => $store_id,
                                    'location_id' => $inventory_location->id,
                                    'variation_id' => $variation->id,
                                ])->first();

                                $inv = \App\Models\Inventory::where(['store_id' => $store_id,'warehouse_id'=>$warehouse->id,'variation_id'=>$variation->id])->first();
                                $old_qty = 0;
                                $new_qty = 0;
                                if($inv) {
                                    $old_qty = $inv['quantity'];
                                    $inv->increment('quantity', $inventory['quantity'], ['updated_at' => $variation->updated_at]);
                                } else {
                                    \App\Models\Inventory::create([
                                        'store_id' => $store_id,
                                        'warehouse_id' => $warehouse->id,
                                        'product_id'=>$product->id,
                                        'variation_id'=>$variation->id,
                                        'quantity' => $inventory['quantity'],
                                        'created_at' => $variation->created_at,
                                        'updated_at' => $variation->updated_at,
                                    ]);
                                }
                                if($spbl) {
                                    $spbl->increment('quantity', $inventory['quantity'], ['updated_at' => $variation->updated_at]);
                                } else {
                                    $spbl = \App\Models\StoreProductCountByLocation::create([
                                        'store_id' => $store_id,
                                        'location_id' => $inventory_location->id,
                                        'product_id' => $product->id,
                                        'variation_id' => $variation->id,
                                        'quantity' => $inventory['quantity'],
                                        'created_at' => $variation->created_at,
                                        'updated_at' => $variation->updated_at,
                                    ]);
                                }
                                $new_qty = $old_qty + $inventory['quantity'];
    
                                \App\Models\StockCard::create([
                                    'store_id' => $store_id,
                                    'warehouse_id' => $warehouse->id,
                                    'product_id' => $product->id,
                                    'variation_id' => $variation->id,
                                    'progress' => 'completed',
                                    'quantity' => $inventory['quantity'],
                                    'quantity_new' => $new_qty,
                                    'quantity_old' => $old_qty,
                                    'type' => 'in',
                                    'remarks' => 'default initial setup',
                                    'created_at' => $variation->created_at,
                                    'updated_at' => $variation->updated_at,
                                ]);

                                
                            }
                        }
                        // if($var->hasFile('image_src')) {
                        //     $image = $var->file('image_src');
                            
                        // }
                    }
                }
            }
            DB::connection()->commit();

            return response()->json($product, 200);
            
        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ], Response::HTTP_UNPROCESSABLE_ENTITY);
        }
    }

    

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, int $id)
    {
        $product = Product::find($id);

        if (!$product) {
            return response()->json(['messages' => 'No data found.'], 202);
        }

        // dd($request->all());
        DB::connection()->beginTransaction();
        try {
            $request = $request->validate([
                'title' => 'string',
                'description' => 'nullable',
                'published' => 'string',
                'published_at' => 'nullable',
                'categories' => 'array',
                'brand_id' => 'string',
                'image' => 'nullable',
                'images' => 'nullable',
                'attributes' => 'array',
            ]);
            dd($request);
            $product->fill($request);
            $product->save();

            $product->touch();
            // dd($request);
            /**if categories is provided */
            // dd($request);
            if($request['categories'] && !empty($request['categories'])) {
                foreach($request['categories'] as $category_id) {
                    ProductCategory::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'category_id' => $category_id
                        ],
                        [
                        'product_id' => $product->id,
                        'category_id' => $category_id,
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
                
            /**if categories is provided */
            if($request['attributes'] && !empty($request['attributes'])) {
                foreach($request['attributes'] as $attribute_id) {
                    ProductAttribute::firstOrCreate(
                        [
                            'product_id' => $product->id,
                            'attribute_id' => $attribute_id,
                        ],
                        [
                        'product_id' => $product->id,
                        'attribute_id' => $attribute_id,
                        'product_id' => $product->id,
                        'attribute_id' => $attribute_id,
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
            
            /**set image path */
            $image_path = 'products';
            /**if main image is uploaded */
            dd($request['image']);
            // dd('has image'.$request->hasFile('image'));
            if ($request->hasFile('image')) {
                dd('hasImage');
                $image = $request->file('image');
                $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                $filePath = Storage::putFileAs($image_path, $image, $imageName);
                $file = File::create([
                    'filename' => $imageName,
                    'disk' => 'storage',
                    'path' => $filePath,
                    'extension' => $image->getClientOriginalExtension(),
                    'size' => $image->getSize(),
                    'mime' => $image->getMimeType(),
                ]);
                // dd($file);
                /**delete previous image */
                ProductImage::where(['product_id'=>$product->id, 'default'=>true])->delete();
                ProductImage::create([
                    'product_id' => $product->id, 
                    'file_id' => $file->id, 
                    'default' => true,
                    'created_at' => $product->created_at,
                    'updated_at' => $product->updated_at,
                ]);
            }
            /**if has uploaded image */
            // dd('has images'.$request->hasFile('images'));
            if ($request->hasFile('images')) {
                foreach($request->file('images') as $image) {
                    $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();
                    $filePath = Storage::putFileAs($image_path, $image, $imageName);
                    $file = File::create([
                        'filename' => $imageName,
                        'disk' => 'storage',
                        'path' => $filePath,
                        'extension' => $image->getClientOriginalExtension(),
                        'size' => $image->getSize(),
                        'mime' => $image->getMimeType(),
                    ]);
                    ProductImage::create([
                        'product_id' => $product->id, 
                        'file_id' => $file->id, 
                        'created_at' => $product->created_at,
                        'updated_at' => $product->updated_at,
                    ]);
                }
            }
            DB::connection()->commit();

            return response()->json($product, 200);

        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ],  Response::HTTP_UNPROCESSABLE_ENTITY);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $data = Product::find($id);

        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }

        DB::connection()->beginTransaction();

        try {
            $data->delete();

            DB::connection()->commit();

            return response()->json([
                'status'  => true,
                'message' => 'Record successfully deleted.',
                'data'    => $data,
            ]);

        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }
}
