<?php

namespace App\Http\Controllers\API\v1;

use App\Models\Role;
use App\Models\RoleStoreAccess;
use App\Models\Store;

use Exception;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Throwable;

class RoleController extends Controller
{

    /**
     * Get search roles
     *
     */
    public function index(Request $request)
    {
        $perPage = $request->input('per_page');

        $keyword = $request->input('keyword', '');

        $data = Role::with(['stores'])
            ->where(function ($query) use ($keyword) {
                $query->where('name', 'like', '%'.$keyword.'%')
            ->orWhere('description', 'like', '%'.$keyword.'%');
        })->orderBy('name')->paginate($perPage);

        return response()->json($data);
    }

    /**
     * Get all role info
     *
     */
    public function show($id)
    {
        return response()->json([
            'status'  => true,
            'message' => 'Role info',
            'data'    => Role::with(['stores'])->where('id', '=', $id)->first()
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => "required|min:2|unique:roles,name",
            'ou' => "required",
            ]
        );


        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            $role = Role::create($request->all());

            //save ROLE store
            $role_stores = [];
            foreach($request->stores as $store) {
                $role_stores[] = array("role_id"=>$role->id, "store_id"=>$store, "created_at"=>$this->current_datetime, "updated_at"=>$this->current_datetime);
            }

            RoleStoreAccess::insert($role_stores);

            DB::connection()->commit();

            return response()->json([
                'status' => true,
                'message' => 'Role saved successfully!',
                'data' => $role
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = Role::find($id);

        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }

        $validator = Validator::make($request->all(), [
            'name' => 'required|min:2|unique:roles,name,'.$data->id
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], Response::HTTP_UNPROCESSABLE_ENTITY);
        }

        DB::connection()->beginTransaction();

        try {

            $data->fill($request->all());
            $data->save();

            $data->touch();

            //find Role in RoleStore
            $roleStoreAccess = RoleStoreAccess::where('role_id', $data->id);
            $roleStoreAccess->delete();

            $role_stores = [];
            foreach($request->stores as $store) {
                $role_stores[] = array("role_id"=>$data->id, "store_id"=>$store, "created_at"=>$data->updated_at, "updated_at"=>$data->updated_at);
            }

            RoleStoreAccess::insert($role_stores);

            DB::connection()->commit();

            return response()->json([
                'status' => true,
                'message' => 'Role updated successfully!',
                'data' => $data
            ]);

        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }


    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        // if ( ! _userAccess('can.delete.roles')) {
        //     return response()->json([
        //         'status' => false,
        //         'message'    => 'Unauthorized.'
        //     ]);
        // }

        $validator = Validator::make(['id' => $id], [
            'id' => 'required|exists:id'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => false,
                'message' => 'Validation error',
                'data' => $validator->errors()
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            $role = Role::find($id);

            $role->delete();

            DB::connection()->commit();

            return response()->json([
                'status' => true,
                'message'    => '"'.$role->name.'" deleted successfully!',
                'data'   => $role
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    public function RoleStoreAccess(Request $request, $id) {

        $data = RoleStoreAccess::select('role_store_access.store_id', 'stores.name')
            ->orderBy('stores.name')
            ->leftJoin('stores', 'role_store_access.store_id', '=', 'stores.id')
            ->where('role_store_access.role_id', $id)
            ->get();
        // if($id != 1) {
        //     $data = $data->toArray();
        // } else {
        //     $data = Store::select('stores.id', 'stores.name')
        //     ->orderBy('stores.name')
        //     ->get();
        //     $data = $data->toArray();
        //     array_unshift($data,['store_id'=>'all','name'=>'ALL']);
        // }
        return response()->json([
            'status' => true,
            'role'   => Role::find($id),
            'data'   => $data
        ]);
    }

    public function AllRoleStoreAccess(Request $request) {
        $role_ids = $request->input('role_ids');
        if(!in_array(1, $role_ids)) {
            $data = RoleStoreAccess::select('stores.id','stores.name')->distinct('stores.name')
                ->orderBy('stores.name')
                ->leftJoin('stores', 'role_store_access.store_id', '=', 'stores.id')
                ->whereIn('role_store_access.role_id', $role_ids)
                ->get();
            /**if has all access get all Stores */
            if(in_array(0,array_column($data->toArray(),'store_id'))) {
                $data = Store::select('stores.*')
                    ->orderBy('stores.name')
                    ->get();
            }
        } else {
            $data = Store::select('stores.*')
                ->orderBy('stores.name')
                ->get();
        }

        return response()->json([
            'status' => true,
            'roles'   => Role::find($role_ids),
            'data'   => $data
        ]);
    }


    /**
     * Get module access assigned to role and store
     *
     */
    public function storePermissions($id, $store_id)
    {
        $data = Role::storePermissions($id, $store_id);
        $data = Arr::pluck($data, 'name');

        return response()->json([
            'status' => true,
            'message'    => 'Assigned Permissions list',
            'data'   => $data
        ]);
    }


}
