<?php

namespace App\Http\Controllers\API\v1;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\Category;
use App\Models\ProductType;
use App\Models\Attribute;
use App\Models\Variation;
use App\Models\Image;
use Illuminate\Http\Request;

class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     */
    public function index()
    {
        /* Get items per page. Default 5 */
        $perPage = request('perPage', 5);
            /* Get records, filter if with search, then paginate and then throw to view page */
            $data = Product::withCount('variations')
                ->orderBy('name')
                ->paginate($perPage);
        return response()->json($data)
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $attrs = $request->validated(); /**Validated inputs */
        if(!empty(request('type'))) /**If type is not empty */
        {
            $type = ProductType::firstOrCreate([
                'name' => request('type') != 'Create New Type'
                ? request('type')
                : request('new_type')
            ]);
            $attrs['product_type_id'] = $type->id;
        }
        $product = Product::create($attrs); /**Create product */
        if(!empty(request('attributes'))) /**if product attributes is not empty */
        {
            $attribs = [];
            foreach(request('attributes') as $attrib)
            {
                $attribute = Attribute::firstOrCreate(['name' => $attrib]);
                array_push($attribs, $attribute->id);
            }
            $product->attributes()->attach($attribs); /**Attach the product attributes */
        }
        if($request->hasFile('product_image')) { /**If product image is not empty */
            $file = $request->file('product_image'); /**get the image */
            $filename = date('YmdHis')."-".$file->getClientOriginalName(); /**get the image filename */
            $file->move(public_path('upload/images'),$filename); /**move the uploaded image to the upload directory */
            $image = Image::create(['file'=>'upload/images/'.$filename]); /**Create new imagerecord */
            $product->images()->sync([$image->id]); /**Sync the product image */
        }
        return response()->json([
            'status' => true,
            'message' => 'Record successfully saved.',
            'data' => $product,
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Masterfile\Product  $product
     * @return \Illuminate\Http\Response
     */
    public function show(Product $product)
    {
        return response()->json([
            'product' => $product,
            'variations' => Variation::with('attributes')->where('product_id',$product->id)->get(),
        ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  $id
     */
    public function edit(int $id)
    {
        $data = Product::find($id);
        return response()->json([
            'product' => $data,
            'categories' => ProductCategory::pluck('name','id')->all(),
            'types' => ProductType::pluck('name')->all(),
            'attributes' => Attribute::pluck('name')->all(),
            'productAttributes' => $product->attributes->pluck('name')->all(),
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  $id
     */
    public function update(Request $request, $id)
    {
        // dd($request);
        $attrs = $request->validated(); /**Validated input */
        $product = Product::find($id);
        if(!empty(request('type'))) /**If type is not empty */
        {
            $type = ProductType::firstOrCreate([
                'name' => request('type') != 'Create New Type'
                ? request('type')
                : request('new_type')
            ]);
            $attrs['product_type_id'] = $type->id;
        }
        else
        {
            $attrs['product_type_id'] = '';
        }
        $product->update($attrs); /**Update the record */
        if(!empty(request('attributes'))) /**if product attributes is not empty */
        {
            $attribs = [];
            foreach(request('attributes') as $attrib)
            {
                $attribute = Attribute::firstOrCreate(['name' => $attrib]);
                array_push($attribs, $attribute->id);
            }
            $product->attributes()->sync($attribs); /**Attach the product attributes */
        }
        else /**else remove the attributes */
        {
            $product->attributes()->detach();
        }
        if ($request->hasFile('product_image')) {
            // $image_path = $request->file('product_image')->store('image', 'public');

            $file = $request->file('product_image'); /**get the image */
            $filename = date('YmdHis')."-".$file->getClientOriginalName(); /**get the image filename */
            $file->move(public_path('upload/images'),$filename); /**move the uploaded image to the upload directory */
            $image = Image::create(['file'=>'upload/images/'.$filename]); /**Create new imagerecord */
            // $image = Image::create(['file'=>$image_path]); /**Create new imagerecord */
            $product->images()->sync([$image->id]); /**Sync the product image */
        }
        /**Redirect to product profile */
        return response()->json([
            'status' => true,
            'message'=> 'Record successfully updated.',
            'data' => $product
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     */
    public function destroy(int $id)
    {
        $product = Product::find($id);
        $product->delete(); /**Delete the product */
        return response()->json([
            'status' => true,
            'message' => 'Record successfully deleted',
        ]);
    }
}
