<?php

namespace App\Http\Controllers\API\v1;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\UserRole;
use App\Models\DefaultStore;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Hash;
use Throwable;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
            $perPage = $request->input('per_page', 10);
            $keyword = $request->input('keyword', '');
            $data = User::with(['roles' => function($query){
                $query->with(['stores']);
            }, 'default_store'])
            ->where(function ($query) use ($keyword) {
                $query->where('email', 'like', '%'.$keyword.'%')
                    ->orWhere('name', 'like', '%'.$keyword.'%')
                    ->orWhere('username', 'like', '%'.$keyword.'%')
                    ;
            })
            ->orderBy('last_name')
            ->orderBy('first_name')
            ->paginate($perPage);
            return response()->json($data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => "required|min:2",
            'username' => "required|min:2|unique:users,username",
            'email' => "required|min:2|email|unique:users,email",
            'first_name' => "required|min:2",
            'last_name' => "required|min:2",
            ]
        );


        if ($validator->fails()) {
            $errors = $validator->errors();
            $first = $errors->getMessages();
            return response()->json([
                'status'  => false,
                'message' => (reset($first))[0],
                'errors'  => $errors
            ]);
        }

        DB::connection()->beginTransaction();

        try {

            $request['password'] = Hash::make('password');
            $request['email_verified_at'] = $this->current_datetime;
            $user = User::create($request->all());
            //save roles
            $roles = [];
            foreach($request->roles as $role) {
                $roles[] = array("user_id"=>$user->id, "role_id"=>$role, "created_at"=>$this->current_datetime, "updated_at"=>$this->current_datetime);
            }

            UserRole::insert($roles);

            DB::connection()->commit();

            return response()->json([
                'status' => true,
                'message' => 'User saved successfully!',
                'data' => $user
            ]);

        } catch (Throwable $e) {

            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $user_id)
    {
        $user = User::find($user_id);
        if($user) {
            return response()->json([
                'status'  => true,
                'data'    => $user
            ]);
        }
        return response()->json(['status'  => false, 'messages' => 'No data found.'], 202);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        $data = User::find($id);

        if (!$data) {
            return response()->json(['messages' => 'No data found.'], 202);
        }
        //if personal info
        if($request['first_name']) {
            $fields = [
                'first_name' => "required|min:2",
                'last_name' => "required|min:2",
                'roles' => "required",
            ];
        } else { //personal info
            $fields = [
                'username' => 'required|min:2|unique:users,username,'.$data->id,
                'email' => 'required|min:2|email|unique:users,email,'.$data->id,
            ];
            if($request['newPassword']) $fields['newPassword'] = 'required|min:6';
        }

        $validator = Validator::make($request->all(), $fields);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], Response::HTTP_UNPROCESSABLE_ENTITY);
        }

        DB::connection()->beginTransaction();

        try {

            $data->fill($request->all());
            $data->save();

            if($request['newPassword']) {
                $data->update(['password' => Hash::make($request['newPassword'])]);
            }
            $data->touch();

            if($request['roles']) {
                //find Role in RoleOU
                $user_role = UserRole::where('user_id', $data->id);
                $user_role->delete();

                $roles = [];
                foreach($request->roles as $role) {
                    $roles[] = array("user_id"=>$data->id, "role_id"=>$role, "created_at"=>$data->updated_at, "updated_at"=>$data->updated_at);
                }

                UserRole::insert($roles);
            }

            DB::connection()->commit();

            return response()->json([
                'status' => true,
                'message' => 'User updated successfully!',
                'data' => $data
            ]);

        } catch (Throwable $e) {
            DB::connection()->rollback();
            return response()->json([
                'status'  => false,
                'message' => 'Unable to process request. Please try again.',
                'data'    => $e->getMessage()
            ]);
        }
    }


    /**
     * Remove the specified resource from storage.
     */
    public function destroy(User $user)
    {
        //
    }
}
